/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 *
 */

package ru.ulstu.controller;

import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import ru.ulstu.HttpUtils;
import ru.ulstu.configuration.ApiConfiguration;
import ru.ulstu.datamodel.ForecastParams;
import ru.ulstu.datamodel.ModelingResult;
import ru.ulstu.datamodel.exception.ModelingException;
import ru.ulstu.datamodel.ts.TimeSeries;
import ru.ulstu.service.MethodParamBruteForce;
import ru.ulstu.service.TimeSeriesService;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.InvocationTargetException;
import java.util.concurrent.ExecutionException;

@RestController
@RequestMapping(ApiConfiguration.API_1_0)
public class TimeSeriesController {
    private final static Logger LOGGER = LoggerFactory.getLogger(TimeSeriesController.class);

    private final TimeSeriesService timeSeriesService;
    private final MethodParamBruteForce methodParamBruteForce;

    public TimeSeriesController(TimeSeriesService timeSeriesService,
                                MethodParamBruteForce methodParamBruteForce) {
        this.timeSeriesService = timeSeriesService;
        this.methodParamBruteForce = methodParamBruteForce;
    }

    @PostMapping("getForecast")
    @ApiOperation("Получить прогноз временного ряда")
    public ResponseEntity<ModelingResult> getForecastTimeSeries(@RequestBody ForecastParams forecastParams, HttpServletRequest request) throws ExecutionException, InterruptedException, InvocationTargetException, NoSuchMethodException, InstantiationException, IllegalAccessException, ModelingException {
        LOGGER.info("User ip: " + HttpUtils.getUserIp(request));
        LOGGER.info("Forecast: " + forecastParams);
        ResponseEntity<ModelingResult> result = new ResponseEntity<>(timeSeriesService.getForecast(forecastParams.getOriginalTimeSeries(),
                forecastParams.getCountForecast()), HttpStatus.OK);
        LOGGER.info("Forecast result complete");
        return result;
    }

    @PostMapping("getSmoothed")
    @ApiOperation("Получить сглаженный временной ряд")
    public ResponseEntity<ModelingResult> getSmoothedTimeSeries(@RequestBody TimeSeries timeSeries, HttpServletRequest request) throws ExecutionException, InterruptedException, InvocationTargetException, NoSuchMethodException, InstantiationException, IllegalAccessException {
        LOGGER.info("User ip: " + HttpUtils.getUserIp(request));
        LOGGER.info("Time series for smoothing: " + timeSeries);
        ResponseEntity<ModelingResult> result = new ResponseEntity<>(methodParamBruteForce.getSmoothedTimeSeries(timeSeries), HttpStatus.OK);
        LOGGER.info("Smoothing complete");
        return result;
    }
}
